package Rcrypt;
use strict;
use Digest::MD5;
use MIME::Base64;

our $VERSION = "1.0";

#####################################################################
# Encode plaintext with a random salt and the secret key
# Plaintext may not contain NULs
# Returns the base 64 encoded ciphertext
sub encrypt
{
    my ($plaintext, $secret) = @_;

    my $salt = pack('n', rand(65535)); # 2 bytes of salt
    my $hash = Digest::MD5::md5($salt . $secret);
    # Replicate the hash until its longer than the plaintext.
    my $hashrep = $hash x int((length($plaintext) + 16) / 16);
    my $encoded = MIME::Base64::encode_base64($salt . ($plaintext ^ $hashrep));
    chomp $encoded; # Strip off trailing newline
    return $encoded;
}

#####################################################################
# Decode the Base64 ciphertext according to the secret key
# Returns the plaintext
sub decrypt
{
    my ($cipher, $secret) = @_;

    my ($salt, $xor) = unpack('a2 a*', MIME::Base64::decode_base64($cipher));
    my $hash = Digest::MD5::md5($salt . $secret);
    # Replicate the hash until its same length as the xored cipher
    # which should be a multiple of 16 bytes
    my $hashrep = $hash x int((length($xor) + 15) / 16);
    my $plaintext = $xor ^ $hashrep;
    # Strip off any NUL padding
    $plaintext =~ s/\000*$//;
    return $plaintext;
}
42;

=head1 NAME

Rcrypt - Rcrypt

=head1 SYNOPSIS

  Test code
  my $secret = 'mysecret';
  foreach ('', 'a', 'fred', '12345678', '123456789012345',
           '1234567890123456', '12345678901234567',
           'jkdfhlukashfouashdfuiahf')
  {
      my $cipher = &encrypt($_, $secret);
      my $plain = &decrypt($cipher, $secret);
      print "cipher $cipher, plain $plain\n";
      print "decode error for $_\n" unless $_ eq $plain;
  }


=head1 DESCRIPTION

 Gepikt van Radius, door Anton

 Oorspronkelijke naam: Radius::Rcrypt

 Encodes/decodes plaintext with a random salt and the secret key
 Plaintext may not contain NULs


=head1 AUTHOR
        Author: Mike McCauley (mikem@open.com.au)
        Anton heeft het package in elkaar geflantst.

=head1 COPYRIGHT

 Geen idee.

=head1 SEE ALSO

perl(1).

=cut




